@@ -59,6 +59,7 @@
 
 int max_threads_glob;
 int set_max_threads_glob=0;
+int const default_max_stored_rows = 256;   
 
 static int enumerate_identity(elpa_index_t index, int i);
 static int cardinality_bool(elpa_index_t index);
@@ -112,6 +113,20 @@ static int min_tile_size_cardinality(elpa_index_t index);
 static int min_tile_size_enumerate(elpa_index_t index, int i);
 static int min_tile_size_is_valid(elpa_index_t index, int n, int new_value);
 
+#ifdef WITH_NVIDIA_GPU_VERSION
+int nvidia_gpu_count();
+#endif
+#ifdef WITH_AMD_GPU_VERSION
+int amd_gpu_count();
+#endif
+#ifdef WITH_INTEL_GPU_VERSION
+//missing function for GPU count
+#endif
+
+static int use_gpu_id_cardinality(elpa_index_t index);
+static int use_gpu_id_enumerate(elpa_index_t index, int i);
+static int use_gpu_id_is_valid(elpa_index_t index, int n, int new_value);
+
 static int valid_with_gpu(elpa_index_t index, int n, int new_value);
 static int valid_with_gpu_elpa1(elpa_index_t index, int n, int new_value);
 static int valid_with_gpu_elpa2(elpa_index_t index, int n, int new_value);
@@ -129,7 +144,7 @@ static int nev_is_valid(elpa_index_t index, int n, int new_value);
 static int bw_is_valid(elpa_index_t index, int n, int new_value);
 static int output_build_config_is_valid(elpa_index_t index, int n, int new_value);
 static int gpu_is_valid(elpa_index_t index, int n, int new_value);
-static int skewsymmetric_is_valid(elpa_index_t index, int n, int new_value);
+static int verbose_is_valid(elpa_index_t index, int n, int new_value);
 
 static int is_positive(elpa_index_t index, int n, int new_value);
 
@@ -139,6 +154,7 @@ static int elpa_float_value_to_string(char *name, float value, const char **stri
 static int elpa_double_string_to_value(char *name, char *string, double *value);
 static int elpa_double_value_to_string(char *name, double value, const char **string);
 
+         
 #define BASE_ENTRY(option_name, option_description, once_value, readonly_value, print_flag_value) \
                 .base = { \
                         .name = option_name, \
@@ -198,11 +214,13 @@ static const elpa_index_int_entry_t int_entries[] = {
         INT_PARAMETER_ENTRY("num_process_rows", "Number of process row number in the 2D domain decomposition", NULL, PRINT_STRUCTURE),
         INT_PARAMETER_ENTRY("num_process_cols", "Number of process column number in the 2D domain decomposition", NULL, PRINT_STRUCTURE),
         INT_PARAMETER_ENTRY("num_processes", "Total number of processes", NULL, PRINT_STRUCTURE),
-        INT_PARAMETER_ENTRY("bandwidth", "If specified, a band matrix with this bandwidth is expected as input; bandwidth must be multiply of nblk", bw_is_valid, PRINT_YES),
+        INT_PARAMETER_ENTRY("bandwidth", "If specified, a band matrix with this bandwidth is expected as input; bandwidth must be multiply of nblk and at least 2", bw_is_valid, PRINT_YES),
         INT_ANY_ENTRY("mpi_comm_rows", "Communicator for inter-row communication", PRINT_NO),
         INT_ANY_ENTRY("mpi_comm_cols", "Communicator for inter-column communication", PRINT_NO),
         INT_ANY_ENTRY("mpi_comm_parent", "Parent communicator", PRINT_NO),
         INT_ANY_ENTRY("blacs_context", "BLACS context", PRINT_NO),
+        INT_ENTRY("verbose", "ELPA API prints verbose messages", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_DOMAIN_ANY, \
+                        cardinality_bool, enumerate_identity, verbose_is_valid, NULL, PRINT_YES),
 //#ifdef REDISTRIBUTE_MATRIX
         INT_ENTRY("internal_nblk", "Internally used block size of scalapack block-cyclic distribution", 0, ELPA_AUTOTUNE_FAST, ELPA_AUTOTUNE_DOMAIN_ANY, \
                    internal_nblk_cardinality, internal_nblk_enumerate, internal_nblk_is_valid, NULL, PRINT_YES),
@@ -215,10 +233,14 @@ static const elpa_index_int_entry_t int_entries[] = {
                          number_of_matrix_layouts, matrix_layout_enumerate, matrix_layout_is_valid, elpa_matrix_layout_name, PRINT_YES), \
         INT_ENTRY("solver", "Solver to use", ELPA_SOLVER_1STAGE, ELPA_AUTOTUNE_FAST, ELPA_AUTOTUNE_DOMAIN_ANY, \
                         number_of_solvers, solver_enumerate, solver_is_valid, elpa_solver_name, PRINT_YES),
-        INT_ENTRY("gpu", "Use GPU acceleration", 0, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_ANY, \
+        INT_ENTRY("gpu", "Use Nvidia GPU acceleration", 0, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_ANY, \
+                        cardinality_bool, enumerate_identity, gpu_is_valid, NULL, PRINT_YES),
+        INT_ENTRY("nvidia-gpu", "Use Nvidia GPU acceleration", 0, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_ANY, \
+                        cardinality_bool, enumerate_identity, gpu_is_valid, NULL, PRINT_YES),
+        INT_ENTRY("intel-gpu", "Use INTEL GPU acceleration", 0, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_ANY, \
+                        cardinality_bool, enumerate_identity, gpu_is_valid, NULL, PRINT_YES),
+        INT_ENTRY("amd-gpu", "Use AMD GPU acceleration", 0, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_ANY, \
                         cardinality_bool, enumerate_identity, gpu_is_valid, NULL, PRINT_YES),
-        INT_ENTRY("is_skewsymmetric", "Matrix is skewsymmetic", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0,
-                        cardinality_bool, enumerate_identity, skewsymmetric_is_valid, NULL, PRINT_YES),
         //default of gpu ussage for individual phases is 1. However, it is only evaluated, if GPU is used at all, which first has to be determined
         //by the parameter gpu and presence of the device
         INT_ENTRY("gpu_tridiag", "Use GPU acceleration for ELPA1 tridiagonalization", 1, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_ANY, \
@@ -236,6 +258,8 @@ static const elpa_index_int_entry_t int_entries[] = {
                         cardinality_bool, enumerate_identity, valid_with_gpu_elpa2, NULL, PRINT_YES),
         INT_ENTRY("gpu_trans_ev_band_to_full", "Use GPU acceleration for ELPA2 trans_ev_band_to_full", 1, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_ANY, \
                         cardinality_bool, enumerate_identity, valid_with_gpu_elpa2, NULL, PRINT_YES),
+	INT_ENTRY("use_gpu_id", "Calling MPI task will use this gpu id", -99, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_DOMAIN_ANY, \
+		  use_gpu_id_cardinality, use_gpu_id_enumerate, use_gpu_id_is_valid, NULL, PRINT_YES), 
         INT_ENTRY("real_kernel", "Real kernel to use if 'solver' is set to ELPA_SOLVER_2STAGE", ELPA_2STAGE_REAL_DEFAULT, ELPA_AUTOTUNE_FAST, ELPA_AUTOTUNE_DOMAIN_REAL, \
                         number_of_real_kernels, real_kernel_enumerate, real_kernel_is_valid, real_kernel_name, PRINT_YES),
         INT_ENTRY("complex_kernel", "Complex kernel to use if 'solver' is set to ELPA_SOLVER_2STAGE", ELPA_2STAGE_COMPLEX_DEFAULT, ELPA_AUTOTUNE_FAST, ELPA_AUTOTUNE_DOMAIN_COMPLEX, \
@@ -253,7 +277,7 @@ static const elpa_index_int_entry_t int_entries[] = {
         INT_ENTRY("stripewidth_complex", "Stripewidth_complex, default 96. Must be a multiple of 8", 96, ELPA_AUTOTUNE_EXTENSIVE, ELPA_AUTOTUNE_DOMAIN_COMPLEX, \
                         stripewidth_complex_cardinality, stripewidth_complex_enumerate, stripewidth_complex_is_valid, NULL, PRINT_YES),
 
-        INT_ENTRY("max_stored_rows", "Maximum number of stored rows used in ELPA 1 backtransformation, default 63", 63, ELPA_AUTOTUNE_EXTENSIVE, ELPA_AUTOTUNE_DOMAIN_ANY, \
+        INT_ENTRY("max_stored_rows", "Maximum number of stored rows used in ELPA 1 backtransformation", default_max_stored_rows, ELPA_AUTOTUNE_EXTENSIVE, ELPA_AUTOTUNE_DOMAIN_ANY, \
                         max_stored_rows_cardinality, max_stored_rows_enumerate, max_stored_rows_is_valid, NULL, PRINT_YES),
 #ifdef WITH_OPENMP_TRADITIONAL
         INT_ENTRY("omp_threads", "OpenMP threads used in ELPA, default 1", 1, ELPA_AUTOTUNE_FAST, ELPA_AUTOTUNE_DOMAIN_ANY, \
@@ -264,13 +288,16 @@ static const elpa_index_int_entry_t int_entries[] = {
 #endif
         INT_ENTRY("cannon_buffer_size", "Increasing the buffer size might make it faster, but costs memory", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_DOMAIN_ANY, \
                         cannon_buffer_size_cardinality, cannon_buffer_size_enumerate, cannon_buffer_size_is_valid, NULL, PRINT_YES),
-        //BOOL_ENTRY("qr", "Use QR decomposition, only used for ELPA_SOLVER_2STAGE, real case", 0, ELPA_AUTOTUNE_MEDIUM, ELPA_AUTOTUNE_DOMAIN_REAL),
+#if defined(THREADING_SUPPORT_CHECK) && defined(ALLOW_THREAD_LIMITING) && !defined(HAVE_SUFFICIENT_MPI_THREADING_SUPPORT)
+        BOOL_ENTRY("limit_openmp_threads", "Limit the number if openmp threads to 1", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_NO),
+#endif
         BOOL_ENTRY("qr", "Use QR decomposition, only used for ELPA_SOLVER_2STAGE, real case", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_DOMAIN_REAL, PRINT_YES),
         BOOL_ENTRY("timings", "Enable time measurement", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
         BOOL_ENTRY("debug", "Emit verbose debugging messages", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
         BOOL_ENTRY("print_flops", "Print FLOP rates on task 0", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
         BOOL_ENTRY("measure_performance", "Also measure with flops (via papi) with the timings", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
         BOOL_ENTRY("check_pd", "Check eigenvalues to be positive", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
+        BOOL_ENTRY("output_pinning_information", "Print the pinning information", 0, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
         BOOL_ENTRY("cannon_for_generalized", "Whether to use Cannons algorithm for the generalized EVP", 1, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
 };
 
@@ -349,8 +376,10 @@ FOR_ALL_TYPES(IMPLEMENT_FIND_ENTRY)
                                 if (elpa_##TYPE##_value_to_string(TYPE##_entries[n].base.name, *value, &value_string) == ELPA_OK) { \
                                         if (!(index->TYPE##_options.notified[n] & notify_flag)) { \
                                                 if (elpa_index_is_printing_mpi_rank(index)) { \
+                                                  if (elpa_index_int_value_is_set(index, "verbose")) { \
                                                         fprintf(stderr, "ELPA: %s '%s' is set to %s due to environment variable %s\n", \
                                                                       error_string, TYPE##_entries[n].base.name, value_string, env_variable); \
+					          } \
                                                 } \
                                                 index->TYPE##_options.notified[n] |= notify_flag; \
                                         } \
@@ -746,17 +775,35 @@ static const char *real_kernel_name(int kernel) {
         }
 }
 
-#define REAL_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
-        kernel_number == ELPA_2STAGE_REAL_GPU ? gpu_is_active : 1
+#define REAL_NVIDIA_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
+        kernel_number == ELPA_2STAGE_REAL_NVIDIA_GPU ? gpu_is_active : 1
+
+#define REAL_AMD_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
+        kernel_number == ELPA_2STAGE_REAL_AMD_GPU ? gpu_is_active : 1
+
+#define REAL_INTEL_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
+        kernel_number == ELPA_2STAGE_REAL_INTEL_GPU ? gpu_is_active : 1
 
 static int real_kernel_is_valid(elpa_index_t index, int n, int new_value) {
         int solver = elpa_index_get_int_value(index, "solver", NULL);
         if (solver == ELPA_SOLVER_1STAGE) {
                 return new_value == ELPA_2STAGE_REAL_DEFAULT;
         }
-        int gpu_is_active = elpa_index_get_int_value(index, "gpu", NULL);
+        int gpu_is_active = (elpa_index_get_int_value(index, "nvidia-gpu", NULL) || elpa_index_get_int_value(index, "gpu", NULL) || elpa_index_get_int_value(index, "amd-gpu", NULL) || elpa_index_get_int_value(index, "intel-gpu", NULL));
         switch(new_value) {
-                ELPA_FOR_ALL_2STAGE_REAL_KERNELS(VALID_CASE_3, REAL_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#ifdef WITH_NVIDIA_GPU_VERSION
+                ELPA_FOR_ALL_2STAGE_REAL_KERNELS(VALID_CASE_3, REAL_NVIDIA_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+#ifdef WITH_AMD_GPU_VERSION
+                ELPA_FOR_ALL_2STAGE_REAL_KERNELS(VALID_CASE_3, REAL_AMD_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+#ifdef WITH_INTEL_GPU_VERSION
+                ELPA_FOR_ALL_2STAGE_REAL_KERNELS(VALID_CASE_3, REAL_INTEL_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+#if !defined(WITH_NVIDIA_GPU_VERSION) && !defined(WITH_AMD_GPU_VERSION) && !defined(WITH_INTEL_GPU_VERSION)
+                ELPA_FOR_ALL_2STAGE_REAL_KERNELS(VALID_CASE_3, REAL_NVIDIA_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+		// intel missing
                 default:
                         return 0;
         }
@@ -785,17 +832,35 @@ static const char *complex_kernel_name(int kernel) {
         }
 }
 
-#define COMPLEX_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
-        kernel_number == ELPA_2STAGE_COMPLEX_GPU ? gpu_is_active : 1
+#define COMPLEX_NVIDIA_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
+        kernel_number == ELPA_2STAGE_COMPLEX_NVIDIA_GPU ? gpu_is_active : 1
+
+#define COMPLEX_AMD_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
+        kernel_number == ELPA_2STAGE_COMPLEX_AMD_GPU ? gpu_is_active : 1
+
+#define COMPLEX_INTEL_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE(kernel_number) \
+        kernel_number == ELPA_2STAGE_COMPLEX_INTEL_GPU ? gpu_is_active : 1
 
 static int complex_kernel_is_valid(elpa_index_t index, int n, int new_value) {
         int solver = elpa_index_get_int_value(index, "solver", NULL);
         if (solver == ELPA_SOLVER_1STAGE) {
                 return new_value == ELPA_2STAGE_COMPLEX_DEFAULT;
         }
-        int gpu_is_active = elpa_index_get_int_value(index, "gpu", NULL);
+        int gpu_is_active = (elpa_index_get_int_value(index, "nvidia-gpu", NULL) || elpa_index_get_int_value(index, "amd-gpu", NULL) || elpa_index_get_int_value(index, "intel-gpu", NULL));
         switch(new_value) {
-                ELPA_FOR_ALL_2STAGE_COMPLEX_KERNELS(VALID_CASE_3, COMPLEX_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#ifdef WITH_NVIDIA_GPU_VERSION
+                ELPA_FOR_ALL_2STAGE_COMPLEX_KERNELS(VALID_CASE_3, COMPLEX_NVIDIA_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+#ifdef WITH_AMD_GPU_VERSION
+                ELPA_FOR_ALL_2STAGE_COMPLEX_KERNELS(VALID_CASE_3, COMPLEX_AMD_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+#ifdef WITH_INTEL_GPU_VERSION
+                ELPA_FOR_ALL_2STAGE_COMPLEX_KERNELS(VALID_CASE_3, COMPLEX_INTEL_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+#if !defined(WITH_NVIDIA_GPU_VERSION) && !defined(WITH_AMD_GPU_VERSION) && !defined(WITH_INTEL_GPU_VERSION)
+                ELPA_FOR_ALL_2STAGE_COMPLEX_KERNELS(VALID_CASE_3, COMPLEX_NVIDIA_GPU_KERNEL_ONLY_WHEN_GPU_IS_ACTIVE)
+#endif
+		// intel missing
                 default:
                         return 0;
         }
@@ -839,7 +904,7 @@ static int bw_is_valid(elpa_index_t index, int n, int new_value) {
         }
 
         na = elpa_index_get_int_value(index, "na", NULL);
-        return (0 <= new_value) && (new_value < na);
+        return (2 <= new_value) && (new_value < na);
 }
 
 static int output_build_config_is_valid(elpa_index_t index, int n, int new_value) {
@@ -850,7 +915,7 @@ static int gpu_is_valid(elpa_index_t index, int n, int new_value) {
         return new_value == 0 || new_value == 1;
 }
 
-static int skewsymmetric_is_valid(elpa_index_t index, int n, int new_value) {
+static int verbose_is_valid(elpa_index_t index, int n, int new_value) {
         return new_value == 0 || new_value == 1;
 }
 
@@ -1027,7 +1092,7 @@ static int omp_threads_is_valid(elpa_index_t index, int n, int new_value) {
 
 
 static int valid_with_gpu(elpa_index_t index, int n, int new_value) {
-        int gpu_is_active = elpa_index_get_int_value(index, "gpu", NULL);
+        int gpu_is_active = (elpa_index_get_int_value(index, "nvidia-gpu", NULL) || elpa_index_get_int_value(index, "gpu", NULL) || elpa_index_get_int_value(index, "amd-gpu", NULL) || elpa_index_get_int_value(index, "intel-gpu", NULL));
         if (gpu_is_active == 1) {
                 return ((new_value == 0 ) || (new_value == 1));
         }
@@ -1038,7 +1103,7 @@ static int valid_with_gpu(elpa_index_t index, int n, int new_value) {
 
 static int valid_with_gpu_elpa1(elpa_index_t index, int n, int new_value) {
         int solver = elpa_index_get_int_value(index, "solver", NULL);
-        int gpu_is_active = elpa_index_get_int_value(index, "gpu", NULL);
+        int gpu_is_active = (elpa_index_get_int_value(index, "nvidia-gpu", NULL) || elpa_index_get_int_value(index, "gpu", NULL) || elpa_index_get_int_value(index, "amd-gpu", NULL) || elpa_index_get_int_value(index, "intel-gpu", NULL));
         if ((solver == ELPA_SOLVER_1STAGE) && (gpu_is_active == 1)) {
                 return ((new_value == 0 ) || (new_value == 1));
         }
@@ -1049,7 +1114,7 @@ static int valid_with_gpu_elpa1(elpa_index_t index, int n, int new_value) {
 
 static int valid_with_gpu_elpa2(elpa_index_t index, int n, int new_value) {
         int solver = elpa_index_get_int_value(index, "solver", NULL);
-        int gpu_is_active = elpa_index_get_int_value(index, "gpu", NULL);
+        int gpu_is_active = (elpa_index_get_int_value(index, "nvidia-gpu", NULL) || elpa_index_get_int_value(index, "gpu", NULL) || elpa_index_get_int_value(index, "amd-gpu", NULL) || elpa_index_get_int_value(index, "intel-gpu", NULL));
         if ((solver == ELPA_SOLVER_2STAGE) && (gpu_is_active == 1)) {
                 return ((new_value == 0 ) || (new_value == 1));
         }
@@ -1059,28 +1124,20 @@ static int valid_with_gpu_elpa2(elpa_index_t index, int n, int new_value) {
 }
 
 static int max_stored_rows_cardinality(elpa_index_t index) {
-	return 8;
+	return 4;
 }
 
 static int max_stored_rows_enumerate(elpa_index_t index, int i) {
-	switch(i) {
-	  case 0:
-	    return 15;
-	  case 1:
-	    return 31;
-	  case 2:
-	    return 47;
-	  case 3:
-	    return 63;
-	  case 4:
-	    return 79;
-	  case 5:
-	    return 95;
-	  case 6:
-	    return 111;
-	  case 7:
-	    return 127;
-	}
+  switch(i) {
+  case 0:
+    return 64;
+  case 1:
+    return 128;
+  case 2:
+    return 256;
+  case 3:
+    return 512;
+  }
 }
 
 static int max_stored_rows_is_valid(elpa_index_t index, int n, int new_value) {
@@ -1092,6 +1149,62 @@ static int max_stored_rows_is_valid(elpa_index_t index, int n, int new_value) {
         }
 }
 
+static int use_gpu_id_cardinality(elpa_index_t index) {
+#ifdef WITH_NVIDIA_GPU_VERSION
+	int count;
+	count = nvidia_gpu_count();
+        if (count == -1000) {
+          fprintf(stderr, "Querrying GPUs failed! Set GPU count = 0\n");
+	return 0;
+        }
+	return count;
+#elif WITH_AMD_GPU_VERION
+	int count;
+	count = amd_gpu_count();
+        if (count == -1000) {
+          fprintf(stderr, "Querrying GPUs failed! Set GPU count = 0\n");
+	return 0;
+        }
+	return count;
+#elif WITH_INTEL_GPU_VERSION
+	return 0;
+#else
+	return 0;
+#endif
+}
+
+static int use_gpu_id_enumerate(elpa_index_t index, int i) {
+        fprintf(stderr, "use_gpu_id_is_enumerate should never be called. please report this bug\n");
+        return i;
+}
+
+static int use_gpu_id_is_valid(elpa_index_t index, int n, int new_value) {
+#ifdef WITH_NVIDIA_GPU_VERSION
+	int count;
+	count = nvidia_gpu_count();
+        if (count == -1000) {
+          fprintf(stderr, "Querrying GPUs failed! Return with error\n");
+	  return 0 == 1 ;
+	} else {
+          return (0 <= new_value) && (new_value <= count);
+	}
+#elif WITH_AMD_GPU_VERSION
+	int count;
+	count = amd_gpu_count();
+        if (count == -1000) {
+          fprintf(stderr, "Querrying GPUs failed! Return with error\n");
+	  return 0 == 1 ;
+	} else {
+          return (0 <= new_value) && (new_value <= count);
+	}
+
+#elif WITH_INTEL_GPU_VERSION
+	return 0 == 1;
+#else
+	return 0 == 0;
+#endif
+
+}
 
 // TODO: this shoudl definitely be improved (too many options to test in autotuning)
 static const int TILE_SIZE_STEP = 128;
@@ -1527,6 +1640,8 @@ int elpa_index_is_printing_mpi_rank(elpa_index_t index)
     process_id = elpa_index_get_int_value(index, "process_id", NULL);
     return (process_id == 0);
   }
-  printf("Warning: process_id not set, printing on all MPI ranks. This can happen with legacy API.");
+  if (elpa_index_int_value_is_set(index, "verbose")) {
+    printf("Warning: process_id not set, printing on all MPI ranks. This can happen with legacy API.");
+  }
   return 1;
 }
